@@ -1,9 +1,12 @@
 import asyncio
+import csv
 import inspect
 import logging
+import re
 import sys
 from datetime import datetime, timedelta
 from functools import wraps
+from io import StringIO
 from os import getenv
 from pathlib import Path
 from uuid import uuid4
@@ -19,7 +22,7 @@ from sqlalchemy.sql import text
 from spellbot._version import __version__
 from spellbot.assets import ASSET_FILES, s
 from spellbot.constants import ADMIN_ROLE, AVG_QUEUE_TIME_WINDOW_MIN, CREATE_ENDPOINT
-from spellbot.data import AuthorizedChannel, Data, Game, Server, Tag, User, WaitTime
+from spellbot.data import Channel, Data, Event, Game, Server, Tag, User, WaitTime
 
 # Application Paths
 RUNTIME_ROOT = Path(".")
@@ -37,6 +40,33 @@ def to_int(s):
         return None
 
 
+def power_and_size_from_params(params):
+    size = 4
+    power = None
+    for param in params:
+        if param.startswith("size:"):
+            size = to_int(param.replace("size:", ""))
+        elif param.startswith("power:"):
+            power = to_int(param.replace("power:", ""))
+    return power, size
+
+
+def tag_names_from_params(params):
+    tag_names = [
+        param
+        for param in params
+        if not param.startswith("size:")
+        and not param.startswith("power:")
+        and not param.startswith("<")
+        and not param.startswith("@")
+        and not param.isdigit()
+        and not len(param) >= 50
+    ]
+    if not tag_names:
+        tag_names = ["default"]
+    return tag_names
+
+
 def is_admin(channel, user_or_member):
     """Checks to see if given user or member has the admin role on this server."""
     member = (
@@ -67,7 +97,7 @@ def paginate(text):
                 break
 
         message = remaining[0 : breakpoint + 1]
-        yield message
+        yield message.rstrip(" >\n")
         remaining = remaining[breakpoint + 1 :]
         last_line_end = message.rfind("\n")
         if last_line_end != -1 and len(message) > last_line_end + 1:
@@ -139,7 +169,7 @@ class SpellBot(discord.Client):
         self.cleanup_started_games_task(loop)
 
     def cleanup_expired_games_task(self, loop):  # pragma: no cover
-        """Starts a task that deletes old games."""
+        """Starts a task that culls old games."""
         THIRTY_SECONDS = 30
 
         async def task():
@@ -150,7 +180,7 @@ class SpellBot(discord.Client):
         loop.create_task(task())
 
     async def cleanup_expired_games(self):
-        """Deletes games older than the given window of minutes."""
+        """Culls games older than the given window of minutes."""
         session = self.data.Session()
         try:
             expired = Game.expired(session)
@@ -161,18 +191,18 @@ class SpellBot(discord.Client):
                         await discord_user.send(s("expired", window=game.server.expire))
                     user.queued_at = None
                     user.game = None
-                game.tags = []
+                game.tags = []  # cascade delete tag associations
                 session.delete(game)
             session.commit()
         except exc.SQLAlchemyError as e:
-            logging.error("error: cleanup_expired_games:", e)
+            logging.exception("error: cleanup_expired_games:", e)
             session.rollback()
             raise
         finally:
             session.close()
 
     def cleanup_expired_wait_times_task(self, loop):  # pragma: no cover
-        """Starts a task that deletes old wait times data."""
+        """Starts a task that culls old wait times data."""
         FIVE_MINUTES = 300
 
         async def task():
@@ -183,41 +213,41 @@ class SpellBot(discord.Client):
         loop.create_task(task())
 
     async def cleanup_expired_waits(self, window):
-        """Deletes wait time data older than the given window of minutes."""
+        """Culls wait time data older than the given window of minutes."""
         session = self.data.Session()
         try:
             cutoff = datetime.utcnow() - timedelta(minutes=window)
             session.query(WaitTime).filter(WaitTime.created_at < cutoff).delete()
             session.commit()
         except exc.SQLAlchemyError as e:
-            logging.error("error: cleanup_expired_waits:", e)
+            logging.exception("error: cleanup_expired_waits:", e)
             session.rollback()
             raise
         finally:
             session.close()
 
     def cleanup_started_games_task(self, loop):  # pragma: no cover
-        """Starts a task that deletes old games."""
-        FIVE_MINUTES = 300
+        """Starts a task that culls old games."""
+        FOUR_HOURS = 14400
 
         async def task():
             while True:
                 await self.cleanup_started_games()
-                await asyncio.sleep(FIVE_MINUTES)
+                await asyncio.sleep(FOUR_HOURS)
 
         loop.create_task(task())
 
     async def cleanup_started_games(self):
-        """Deletes games older than the given window of minutes."""
+        """Culls games older than the given window of minutes."""
         session = self.data.Session()
         try:
-            games = session.query(Game).filter(Game.url != None).all()
+            games = session.query(Game).filter(Game.status == "started").all()
             for game in games:
-                game.tags = []
+                game.tags = []  # cascade delete tag associations
                 session.delete(game)
             session.commit()
         except exc.SQLAlchemyError as e:
-            logging.error("error: cleanup_started_games:", e)
+            logging.exception("error: cleanup_started_games:", e)
             session.rollback()
             raise
         finally:
@@ -236,7 +266,7 @@ class SpellBot(discord.Client):
 
     def ensure_user_exists(self, user):
         """Ensures that the user row exists for the given discord user."""
-        db_user = self.session.query(User).filter(User.xid == user.id).first()
+        db_user = self.session.query(User).filter(User.xid == user.id).one_or_none()
         if not db_user:
             db_user = User(xid=user.id)
             self.session.add(db_user)
@@ -244,7 +274,9 @@ class SpellBot(discord.Client):
 
     def ensure_server_exists(self, guild_xid):
         """Ensures that the server row exists for the given discord guild id."""
-        server = self.session.query(Server).filter(Server.guild_xid == guild_xid).first()
+        server = (
+            self.session.query(Server).filter(Server.guild_xid == guild_xid).one_or_none()
+        )
         if not server:
             server = Server(guild_xid=guild_xid)
             self.session.add(server)
@@ -274,22 +306,15 @@ class SpellBot(discord.Client):
             method = getattr(self, command)
             if not method.allow_dm and str(message.channel.type) == "private":
                 return await message.author.send(s("no_dm"))
-            mentions = message.mentions if message.channel.type != "private" else []
             logging.debug(
-                "%s%s (channel=%s, author=%s, mentions=%s, params=%s)",
-                prefix,
-                command,
-                message.channel,
-                message.author,
-                mentions,
-                params,
+                "%s%s (params=%s, message=%s)", prefix, command, params, message
             )
             self.session = self.data.Session()
             try:
-                await method(prefix, message.channel, message.author, mentions, params)
+                await method(prefix, params, message)
                 self.session.commit()
             except exc.SQLAlchemyError as e:
-                logging.error(f"error: {request}:", e)
+                logging.exception(f"error: {request}:", e)
                 self.session.rollback()
                 raise
             finally:
@@ -352,12 +377,12 @@ class SpellBot(discord.Client):
     # bot command. These methods should have a signature like:
     #
     #     @command(allow_dm=True)
-    #     def command_name(self, channel, author,mentions, params)
+    #     def command_name(self, prefix, params, message)
     #
     # - `allow_dm` indicates if the command is allowed to be used in direct messages.
-    # - `channel` is the Discord channel where the command message was sent.
-    # - `author` is the Discord author who sent the command.
+    # - `prefix` is the command prefix, which is "!" by default.
     # - `params` are any space delimitered parameters also sent with the command.
+    # - `message` is the discord.py message object that triggered the command.
     #
     # The docstring used for the command method will be automatically used as the help
     # message for the command. To document commands with parameters use a & to delimit
@@ -368,7 +393,7 @@ class SpellBot(discord.Client):
     # Where [foo] indicates foo is optional and <bar> indicates bar is required.
 
     @command(allow_dm=True)
-    async def help(self, prefix, channel, author, mentions, params):
+    async def help(self, prefix, params, message):
         """
         Sends you this help message.
         """
@@ -378,7 +403,19 @@ class SpellBot(discord.Client):
             doc = method.__doc__.split("&")
             use, params = doc[0], ", ".join([param.strip() for param in doc[1:]])
             use = inspect.cleandoc(use)
+
+            transformed = ""
+            for line in use.split("\n"):
+                if line:
+                    if line.startswith("*"):
+                        transformed += f"\n{line}"
+                    else:
+                        transformed += f"{line} "
+                else:
+                    transformed += "\n\n"
+            use = transformed
             use = use.replace("\n", "\n> ")
+            use = re.sub(r"([^>])\s+$", r"\1", use, flags=re.M)
 
             title = f"{prefix}{command}"
             if params:
@@ -396,12 +433,12 @@ class SpellBot(discord.Client):
             "\n💜 You can help keep SpellBot running by supporting me on Ko-fi! "
             "<https://ko-fi.com/Y8Y51VTHZ>"
         )
-        await channel.send(s("dm_sent"))
+        await message.channel.send(s("dm_sent"))
         for page in paginate(usage):
-            await author.send(page)
+            await message.author.send(page)
 
     @command(allow_dm=True)
-    async def about(self, prefix, channel, author, mentions, params):
+    async def about(self, prefix, params, message):
         """
         Get information about SpellBot.
         """
@@ -430,10 +467,10 @@ class SpellBot(discord.Client):
         embed.url = "https://github.com/lexicalunit/spellbot"
         embed.set_footer(text="MIT © amy@lexicalunit et al")
         embed.color = discord.Color(0x5A3EFD)
-        await channel.send(embed=embed, file=None)
+        await message.channel.send(embed=embed, file=None)
 
     @command(allow_dm=False)
-    async def play(self, prefix, channel, author, mentions, params):
+    async def play(self, prefix, params, message):
         """
         Enter a play queue for a game on SpellTable.
 
@@ -458,50 +495,36 @@ class SpellBot(discord.Client):
         same channel as you did. The default scope for matchmaking is server-wide.
         & [@mention-1] [@mention-2] [...] [size:N] [power:N] [tag-1] [tag-2] [...]
         """
-        params = [param.lower() for param in params]
-        user = self.ensure_user_exists(author)
+        user = self.ensure_user_exists(message.author)
         if user.waiting:
-            return await channel.send(s("play_already"))
+            return await message.channel.send(s("play_already"))
 
-        size = 4
-        power = None
-        for param in params:
-            if param.startswith("size:"):
-                size = to_int(param.replace("size:", ""))
-            elif param.startswith("power:"):
-                power = to_int(param.replace("power:", ""))
+        params = [param.lower() for param in params]
+        mentions = message.mentions if message.channel.type != "private" else []
 
+        power, size = power_and_size_from_params(params)
         if not size or not (1 < size < 5):
-            return await channel.send(s("play_size_bad"))
-
+            return await message.channel.send(s("play_size_bad"))
         if power and not (1 <= power <= 10):
-            return await channel.send(s("play_power_bad"))
+            return await message.channel.send(s("play_power_bad"))
 
-        if len(mentions) >= size:
-            return await channel.send(s("play_too_many_mentions"))
+        if len(mentions) + 1 > size:
+            return await message.channel.send(s("play_too_many_mentions"))
 
         mentioned_users = []
         for mentioned in mentions:
             mentioned_user = self.ensure_user_exists(mentioned)
             if mentioned_user.waiting:
-                return await channel.send(s("play_mention_already", user=mentioned))
+                return await message.channel.send(
+                    s("play_mention_already", user=mentioned)
+                )
             mentioned_users.append(mentioned_user)
 
-        tags = []
-        tag_names = [
-            param
-            for param in params
-            if not param.startswith("size:")
-            and not param.startswith("power:")
-            and not param.startswith("<")
-            and not param.startswith("@")
-            and not param.isdigit()
-            and not len(param) >= 50
-        ]
-        if not tag_names:
-            tag_names = ["default"]
+        tag_names = tag_names_from_params(params)
         if len(tag_names) > 5:
-            return await channel.send(s("play_too_many_tags"))
+            return await message.channel.send(s("play_too_many_tags"))
+
+        tags = []
         for tag_name in tag_names:
             tag = self.session.query(Tag).filter_by(name=tag_name).first()
             if not tag:
@@ -509,10 +532,12 @@ class SpellBot(discord.Client):
                 self.session.add(tag)
             tags.append(tag)
 
-        server = self.ensure_server_exists(channel.guild.id)
+        server = self.ensure_server_exists(message.channel.guild.id)
+        self.session.commit()
+
         user.enqueue(
             server=server,
-            channel_xid=channel.id,
+            channel_xid=message.channel.id,
             include=mentioned_users,
             size=size,
             power=power,
@@ -529,9 +554,10 @@ class SpellBot(discord.Client):
                 else:
                     found_discord_users.append(discord_user)
 
-        await channel.send(s("dm_sent"))
+        await message.channel.send(s("dm_sent"))
         if len(found_discord_users) == size:  # all players matched, game is ready
             user.game.url = self.create_game()
+            user.game.status = "started"
             game_created_at = datetime.utcnow()
             response = user.game.to_str()
             for game_user, discord_user in zip(user.game.users, found_discord_users):
@@ -539,138 +565,390 @@ class SpellBot(discord.Client):
                 WaitTime.log(
                     self.session,
                     guild_xid=server.guild_xid,
-                    channel_xid=channel.id,
+                    channel_xid=message.channel.id,
                     seconds=(game_created_at - game_user.queued_at).total_seconds(),
                 )
                 game_user.queued_at = None
         else:  # still waiting on more players, game is pending
             response = user.game.to_str()
-            await author.send(response)
+            await message.author.send(response)
             for mention in mentions:
                 await mention.send(response)
 
+    @command(allow_dm=False)
+    async def event(self, prefix, params, message):
+        """
+        Create many games in batch from an attached CSV data file. _Requires the
+        "SpellBot Admin" role._
+
+        For example, if your event is for a Modern tournement you might attach a CSV file
+        with a comment like `!event Player1Username Player2Username`. This would assume
+        that the players' discord user names are found in the "Player1Username" and
+        "Player2Username" CSV columns. The game size is deduced from the number of column
+        names given, so we know the games created in this example are `size:2`.
+
+        Games will not be created immediately to allow you to verify things look ok first.
+        After the event is created you will be given directions on how to start it.
+        & <player 1 column> <player 2 column> ... <player N column>
+        """
+        if not is_admin(message.channel, message.author):
+            return await message.channel.send(s("not_admin"))
+
+        if not message.attachments:
+            return await message.channel.send(s("event_no_data"))
+
+        if not params:
+            return await message.channel.send(s("event_no_params"))
+
+        size = len(params)
+        if not (1 < size <= 4):
+            return await message.channel.send(s("event_bad_play_count"))
+
+        attachment = message.attachments[0]
+
+        if not attachment.filename.lower().endswith(".csv"):
+            return await message.channel.send(s("event_not_csv"))
+
+        bdata = await message.attachments[0].read()
+        sdata = bdata.decode("utf-8")
+
+        has_header = csv.Sniffer().has_header(sdata)
+        if not has_header:
+            return await message.channel.send(s("event_no_header"))
+
+        server = self.ensure_server_exists(message.channel.guild.id)
+        reader = csv.reader(StringIO(sdata))
+        header = [column.lower().strip() for column in next(reader)]
+        params = [param.lower().strip() for param in params]
+
+        if any(param not in header for param in params):
+            return await message.channel.send(s("event_no_header"))
+
+        columns = [header.index(param) for param in params]
+
+        event = Event()
+        self.session.add(event)
+        self.session.commit()
+
+        members = message.channel.guild.members
+        member_lookup = {member.name.lower().strip(): member for member in members}
+        for i, row in enumerate(reader):
+            values = [row[column] for column in columns]
+            players_s = ", ".join([f'"{value}"' for value in values])
+            player_names = [value.lower().strip() for value in values]
+
+            if any(not player_name for player_name in player_names):
+                warning = s("event_missing_player", row=i + 1, players=players_s)
+                await message.channel.send(warning)
+                continue
+
+            player_discord_users = []
+            for player_name in player_names:
+                player_discord_user = member_lookup.get(player_name)
+                if player_discord_user:
+                    player_discord_users.append(player_discord_user)
+                else:
+                    warning = s(
+                        "event_missing_user",
+                        row=i + 1,
+                        name=player_name,
+                        players=players_s,
+                    )
+                    await message.channel.send(warning)
+
+            if len(player_discord_users) != size:
+                continue
+
+            player_users = [
+                self.ensure_user_exists(player_discord_user)
+                for player_discord_user in player_discord_users
+            ]
+
+            for player_user in player_users:
+                if player_user.waiting:
+                    player_user.dequeue()
+            self.session.commit()
+
+            now = datetime.utcnow()
+            expires_at = now + timedelta(minutes=server.expire)
+            game = Game(
+                created_at=now,
+                expires_at=expires_at,
+                guild_xid=message.channel.guild.id,
+                size=size,
+                updated_at=now,
+                status="ready",
+                users=player_users,
+                event=event,
+            )
+            self.session.add(game)
+            self.session.commit()
+
+        if not event.games:
+            self.session.delete(event)
+            return await message.channel.send(s("event_empty"))
+
+        await message.channel.send(s("event_created", prefix=prefix, event_id=event.id))
+
+    @command(allow_dm=False)
+    async def start(self, prefix, params, message):
+        """
+        Confirm creation of games for the given event id. _Requires the
+        "SpellBot Admin" role._
+        & <event id>
+        """
+        if not is_admin(message.channel, message.author):
+            return await message.channel.send(s("not_admin"))
+
+        if not params:
+            return await message.channel.send(s("start_no_params"))
+
+        event_id = to_int(params[0])
+        if not event_id:
+            return await message.channel.send(s("start_bad_event"))
+
+        event = self.session.query(Event).filter(Event.id == event_id).one_or_none()
+        if not event:
+            return await message.channel.send(s("start_bad_event"))
+
+        if event.started:
+            return await message.channel.send(s("start_event_already_started"))
+
+        for game in event.games:
+            players_str = ", ".join(sorted([f"<@{user.xid}>" for user in game.users]))
+
+            found_discord_users = []
+            for game_user in game.users:
+                discord_user = self.get_user(game_user.xid)
+                if not discord_user:  # game_user has left the server since event created
+                    warning = s("start_user_left", players=players_str)
+                    await message.channel.send(warning)
+                else:
+                    found_discord_users.append(discord_user)
+            if len(found_discord_users) != len(game.users):
+                continue
+
+            game.url = self.create_game()
+            game.status = "started"
+            response = game.to_str()
+            for discord_user in found_discord_users:
+                await discord_user.send(response)
+
+            await message.channel.send(
+                s("game_created", id=game.id, url=game.url, players=players_str)
+            )
+
+    @command(allow_dm=False)
+    async def game(self, prefix, params, message):
+        """
+        Create a game between mentioned users. _Requires the "SpellBot Admin" role._
+
+        Operates similarly to the `!play` command with a few key deferences. First, see
+        that command's usage help for more details. Then, here are the differences:
+        * The user who issues this command is **NOT** added to the game themselves.
+        * You must mention all of the players to be seated in the game.
+        * Optional: Add a message by using " -- " followed by the message content.
+        & [similar parameters as !play] [-- An optional additional message to send.]
+        """
+        if not is_admin(message.channel, message.author):
+            return await message.channel.send(s("not_admin"))
+
+        optional_message = None
+        try:
+            sentry = params.index("--")
+            optional_message = " ".join(params[sentry + 1 :])
+            params = params[0:sentry]
+        except ValueError:
+            pass
+
+        if optional_message and len(optional_message) >= 255:
+            return await message.channel.send(s("game_message_too_long"))
+
+        params = [param.lower() for param in params]
+        mentions = message.mentions if message.channel.type != "private" else []
+
+        power, size = power_and_size_from_params(params)
+        if not size or not (1 < size <= 4):
+            return await message.channel.send(s("game_size_bad"))
+        if power and not (1 <= power <= 10):
+            return await message.channel.send(s("game_power_bad"))
+
+        if len(mentions) > size:
+            return await message.channel.send(s("game_too_many_mentions", size=size))
+        elif len(mentions) < size:
+            return await message.channel.send(s("game_too_few_mentions", size=size))
+
+        mentioned_users = []
+        for mentioned in mentions:
+            mentioned_user = self.ensure_user_exists(mentioned)
+            if mentioned_user.waiting:
+                mentioned_user.dequeue()
+            mentioned_users.append(mentioned_user)
+        self.session.commit()
+
+        tag_names = tag_names_from_params(params)
+        if len(tag_names) > 5:
+            return await message.channel.send(s("game_too_many_tags"))
+
+        tags = []
+        for tag_name in tag_names:
+            tag = self.session.query(Tag).filter_by(name=tag_name).first()
+            if not tag:
+                tag = Tag(name=tag_name)
+                self.session.add(tag)
+            tags.append(tag)
+
+        server = self.ensure_server_exists(message.channel.guild.id)
+        self.session.commit()
+
+        now = datetime.utcnow()
+        expires_at = now + timedelta(minutes=server.expire)
+        url = self.create_game()
+        game = Game(
+            channel_xid=message.channel.id if server.scope == "channel" else None,
+            created_at=now,
+            expires_at=expires_at,
+            guild_xid=server.guild_xid,
+            power=power,
+            size=size,
+            updated_at=now,
+            url=url,
+            status="started",
+            message=optional_message,
+            users=mentioned_users,
+            tags=tags,
+        )
+        self.session.add(game)
+        self.session.commit()
+
+        player_response = game.to_str()
+        for player in mentioned_users:
+            discord_user = self.get_user(player.xid)
+            await discord_user.send(player_response)
+            player.queued_at = None
+
+        players_str = ", ".join(sorted([f"<@{user.xid}>" for user in mentioned_users]))
+        await message.channel.send(
+            s("game_created", id=game.id, url=game.url, players=players_str)
+        )
+
     @command(allow_dm=True)
-    async def leave(self, prefix, channel, author, mentions, params):
+    async def leave(self, prefix, params, message):
         """
         Leave your place in the queue.
         """
-        user = self.ensure_user_exists(author)
+        user = self.ensure_user_exists(message.author)
         if not user.waiting:
-            return await channel.send(s("leave_already"))
+            return await message.channel.send(s("leave_already"))
 
         user.dequeue()
-        await channel.send(s("leave"))
+        await message.channel.send(s("leave"))
 
     @command(allow_dm=False)
-    async def status(self, prefix, channel, author, mentions, params):
+    async def status(self, prefix, params, message):
         """
         Show some details about the queues on your server.
         """
-        server = self.ensure_server_exists(channel.guild.id)
+        server = self.ensure_server_exists(message.channel.guild.id)
         average = WaitTime.average(
             self.session,
             guild_xid=server.guild_xid,
-            channel_xid=channel.id,
+            channel_xid=message.channel.id,
             scope=server.scope,
             window_min=AVG_QUEUE_TIME_WINDOW_MIN,
         )
         if average:
             wait = naturaldelta(timedelta(seconds=average))
-            await channel.send(s("status", wait=wait))
+            await message.channel.send(s("status", wait=wait))
         else:
-            await channel.send(s("status_unknown"))
+            await message.channel.send(s("status_unknown"))
 
     @command(allow_dm=False)
-    async def spellbot(self, prefix, channel, author, mentions, params):
+    async def spellbot(self, prefix, params, message):
         """
-        Configure SpellBot for your server.
+        Configure SpellBot for your server. _Requires the "SpellBot Admin" role._
 
         The following subcommands are supported:
-
         * `config`: Just show the current configuration for this server.
         * `channel <list>`: Set SpellBot to only respond in the given list of channels.
         * `prefix <string>`: Set SpellBot prefix for commands in text channels.
         * `scope <server|channel>`: Set matchmaking scope to server-wide or channel-only.
         * `expire <number>`: Set the number of minutes before pending games expire.
-
-        _You must have the "SpellBot Admin" role to use any of these commands._
         & <subcommand> [subcommand parameters]
         """
-        if not is_admin(channel, author):
-            return await channel.send(s("not_admin"))
+        if not is_admin(message.channel, message.author):
+            return await message.channel.send(s("not_admin"))
         if not params:
-            return await channel.send(s("spellbot_missing_subcommand"))
-        self.ensure_server_exists(channel.guild.id)
+            return await message.channel.send(s("spellbot_missing_subcommand"))
+        self.ensure_server_exists(message.channel.guild.id)
         command = params[0]
         if command == "channels":
-            await self.spellbot_channels(prefix, channel, author, mentions, params[1:])
+            await self.spellbot_channels(prefix, params[1:], message)
         elif command == "prefix":
-            await self.spellbot_prefix(prefix, channel, author, mentions, params[1:])
+            await self.spellbot_prefix(prefix, params[1:], message)
         elif command == "scope":
-            await self.spellbot_scope(prefix, channel, author, mentions, params[1:])
+            await self.spellbot_scope(prefix, params[1:], message)
         elif command == "expire":
-            await self.spellbot_expire(prefix, channel, author, mentions, params[1:])
+            await self.spellbot_expire(prefix, params[1:], message)
         elif command == "config":
-            await self.spellbot_config(prefix, channel, author, mentions, params[1:])
+            await self.spellbot_config(prefix, params[1:], message)
         else:
-            await channel.send(s("spellbot_unknown_subcommand", command=command))
+            await message.channel.send(s("spellbot_unknown_subcommand", command=command))
 
-    async def spellbot_channels(self, prefix, channel, author, mentions, params):
+    async def spellbot_channels(self, prefix, params, message):
         if not params:
-            return await channel.send(s("spellbot_channels_none"))
-        self.session.query(AuthorizedChannel).filter_by(
-            guild_xid=channel.guild.id
-        ).delete()
+            return await message.channel.send(s("spellbot_channels_none"))
+        self.session.query(Channel).filter_by(guild_xid=message.channel.guild.id).delete()
         for param in params:
-            self.session.add(AuthorizedChannel(guild_xid=channel.guild.id, name=param))
-        return await channel.send(
+            self.session.add(Channel(guild_xid=message.channel.guild.id, name=param))
+        await message.channel.send(
             s("spellbot_channels", channels=", ".join([f"#{param}" for param in params]))
         )
 
-    async def spellbot_prefix(self, prefix, channel, author, mentions, params):
+    async def spellbot_prefix(self, prefix, params, message):
         if not params:
-            return await channel.send(s("spellbot_prefix_none"))
+            return await message.channel.send(s("spellbot_prefix_none"))
         prefix_str = params[0][0:10]
         server = (
             self.session.query(Server)
-            .filter(Server.guild_xid == channel.guild.id)
+            .filter(Server.guild_xid == message.channel.guild.id)
             .one_or_none()
         )
         server.prefix = prefix_str
-        return await channel.send(s("spellbot_prefix", prefix=prefix_str))
+        return await message.channel.send(s("spellbot_prefix", prefix=prefix_str))
 
-    async def spellbot_scope(self, prefix, channel, author, mentions, params):
+    async def spellbot_scope(self, prefix, params, message):
         if not params:
-            return await channel.send(s("spellbot_scope_none"))
+            return await message.channel.send(s("spellbot_scope_none"))
         scope_str = params[0].lower()
         if scope_str not in ("server", "channel"):
-            return await channel.send(s("spellbot_scope_bad"))
+            return await message.channel.send(s("spellbot_scope_bad"))
         server = (
             self.session.query(Server)
-            .filter(Server.guild_xid == channel.guild.id)
+            .filter(Server.guild_xid == message.channel.guild.id)
             .one_or_none()
         )
         server.scope = scope_str
-        return await channel.send(s("spellbot_scope", scope=scope_str))
+        await message.channel.send(s("spellbot_scope", scope=scope_str))
 
-    async def spellbot_expire(self, prefix, channel, author, mentions, params):
+    async def spellbot_expire(self, prefix, params, message):
         if not params:
-            return await channel.send(s("spellbot_expire_none"))
+            return await message.channel.send(s("spellbot_expire_none"))
         expire = to_int(params[0])
         if not expire or not (0 < expire <= 60):
-            return await channel.send(s("spellbot_expire_bad"))
+            return await message.channel.send(s("spellbot_expire_bad"))
         server = (
             self.session.query(Server)
-            .filter(Server.guild_xid == channel.guild.id)
+            .filter(Server.guild_xid == message.channel.guild.id)
             .one_or_none()
         )
         server.expire = expire
-        return await channel.send(s("spellbot_expire", expire=expire))
+        await message.channel.send(s("spellbot_expire", expire=expire))
 
-    async def spellbot_config(self, prefix, channel, author, mentions, params):
+    async def spellbot_config(self, prefix, params, message):
         server = (
             self.session.query(Server)
-            .filter(Server.guild_xid == channel.guild.id)
+            .filter(Server.guild_xid == message.channel.guild.id)
             .one_or_none()
         )
         embed = discord.Embed(title="SpellBot Server Config")
@@ -691,7 +969,7 @@ class SpellBot(discord.Client):
         embed.add_field(name="Authorized channels", value=channels_str)
         embed.color = discord.Color(0x5A3EFD)
         embed.set_footer(text=f"Config for Guild ID: {server.guild_xid}")
-        await channel.send(embed=embed, file=None)
+        await message.channel.send(embed=embed, file=None)
 
 
 def get_db_env(fallback):  # pragma: no cover
@@ -766,12 +1044,16 @@ def main(
 
     token = getenv("SPELLBOT_TOKEN", None)
     if not token:
-        print("error: SPELLBOT_TOKEN environment variable not set", file=sys.stderr)
+        print(  # noqa: T001
+            "error: SPELLBOT_TOKEN environment variable not set", file=sys.stderr
+        )
         sys.exit(1)
 
     auth = getenv("SPELLTABLE_AUTH", None)
-    if not auth:
-        print("error: SPELLTABLE_AUTH environment variable not set", file=sys.stderr)
+    if not auth and not mock_games:
+        print(  # noqa: T001
+            "error: SPELLTABLE_AUTH environment variable not set", file=sys.stderr
+        )
         sys.exit(1)
 
     client = SpellBot(
