@@ -1,3 +1,4 @@
+import json
 from datetime import datetime, timedelta
 from pathlib import Path
 
@@ -69,10 +70,20 @@ class Server(Base):
     scope = Column(String(10), nullable=False, default="server")
     expire = Column(Integer, nullable=False, server_default=text("30"))  # minutes
     games = relationship("Game", back_populates="server")
-    authorized_channels = relationship("AuthorizedChannel", back_populates="server")
+    authorized_channels = relationship("Channel", back_populates="server")
+
+    def __repr__(self):
+        return json.dumps(
+            {
+                "guild_xid": self.guild_xid,
+                "prefix": self.prefix,
+                "scope": self.scope,
+                "expire": self.expire,
+            }
+        )
 
 
-class AuthorizedChannel(Base):
+class Channel(Base):
     __tablename__ = "authorized_channels"
     id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
     guild_xid = Column(
@@ -90,10 +101,22 @@ games_tags = Table(
 )
 
 
+class Event(Base):
+    __tablename__ = "events"
+    id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
+    games = relationship("Game", back_populates="event")
+
+    @property
+    def started(self):
+        return any(game.status == "started" for game in self.games)
+
+    def __repr__(self):
+        return json.dumps({"id": self.id})
+
+
 class User(Base):
     __tablename__ = "users"
-    id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
-    xid = Column(BigInteger, nullable=False)
+    xid = Column(BigInteger, primary_key=True, nullable=False)
     game_id = Column(Integer, ForeignKey("games.id", ondelete="SET NULL"), nullable=True)
     queued_at = Column(DateTime, nullable=True)
     game = relationship("Game", back_populates="users")
@@ -106,46 +129,35 @@ class User(Base):
         session = Session.object_session(self)
         guild_xid = server.guild_xid
         required_tag_ids = set(tag.id for tag in tags)
+        filters = [
+            games_tags.c.tag_id.in_([tag.id for tag in tags]),
+            Game.status == "pending",
+            Game.guild_xid == guild_xid,
+            Game.size == size,
+        ]
+        if server.scope == "channel":
+            filters.append(Game.channel_xid == channel_xid)
+        if power:
+            filters.append(Game.power >= power - 2)
+            filters.append(Game.power <= power + 2)
         considerations = (
             session.query(games_tags.c.game_id, func.count(games_tags.c.game_id))
-            .filter(games_tags.c.tag_id.in_([tag.id for tag in tags]))
+            .join(Game)
+            .filter(and_(*filters))
             .group_by(games_tags.c.game_id)
             .having(func.count(games_tags.c.game_id) == len(tags))
             .all()
         )
-        valid_game_ids = []
+        existing_game = None
         for row in considerations:
+            # FIXME: How can we move these checks into the query statement?
             game = session.query(Game).get(row.game_id)
-            if not game:
-                continue
-            if game.url is not None:
-                continue
             if set(tag.id for tag in game.tags) != required_tag_ids:
                 continue
             if len(game.users) >= size - len(include):
                 continue
-            if game.guild_xid != guild_xid:
-                continue
-            if game.size != size:
-                continue
-            if (
-                server.scope == "channel"
-                and game.channel_xid
-                and game.channel_xid != channel_xid
-            ):
-                continue
-            if power:
-                if not game.power:
-                    continue
-                if not (power - 2 <= game.power <= power + 2):
-                    continue
-            valid_game_ids.append(row.game_id)
-        existing_game = (
-            session.query(Game)
-            .filter(Game.id.in_(valid_game_ids))
-            .order_by(Game.created_at)
-            .first()
-        )
+            existing_game = game
+            break
         now = datetime.utcnow()
         expires_at = now + timedelta(minutes=server.expire)
         if existing_game:
@@ -154,15 +166,11 @@ class User(Base):
             self.game.updated_at = now
             self.game.expires_at = expires_at
             if power:
+                # recalculating the average power for this game
                 newly_seated = 1 + len(include)
                 now_seated = already_seated + newly_seated
-                print(
-                    f"power: (({self.game.power} * {already_seated}) +"
-                    f" ({power} * {newly_seated})) / {now_seated} "
-                )
-                self.game.power = (
-                    (self.game.power * already_seated) + (power * newly_seated)
-                ) / now_seated
+                total_power = self.game.power * already_seated + power * newly_seated
+                self.game.power = total_power / now_seated
         else:
             self.game = Game(
                 channel_xid=channel_xid if server.scope == "channel" else None,
@@ -183,6 +191,7 @@ class User(Base):
     def dequeue(self):
         session = Session.object_session(self)
         if self.game and len(self.game.users) == 1:
+            self.game.tags = []  # cascade delete tag associations
             session.delete(self.game)
         self.game = None
         self.queued_at = None
@@ -201,23 +210,55 @@ class Game(Base):
     channel_xid = Column(BigInteger)
     power = Column(Float)
     url = Column(String(255))
+    status = Column(String(30), nullable=False, server_default=text("'pending'"))
+    message = Column(String(255))
+    event_id = Column(
+        Integer, ForeignKey("events.id", ondelete="SET NULL"), nullable=True
+    )
     users = relationship("User", back_populates="game")
     tags = relationship("Tag", secondary=games_tags, back_populates="games")
     server = relationship("Server", back_populates="games")
+    event = relationship("Event", back_populates="games")
 
     @classmethod
     def expired(cls, session):
         return (
             session.query(Game)
-            .filter(and_(datetime.utcnow() >= Game.expires_at, Game.url == None))
+            .filter(
+                and_(
+                    datetime.utcnow() >= Game.expires_at,
+                    Game.url == None,
+                    Game.status != "ready",
+                )
+            )
             .all()
         )
 
+    def __repr__(self):
+        return json.dumps(
+            {
+                "id": self.id,
+                "created_at": str(self.created_at),
+                "updated_at": str(self.updated_at),
+                "expires_at": str(self.expires_at),
+                "size": self.size,
+                "guild_xid": self.guild_xid,
+                "channel_xid": self.channel_xid,
+                "power": self.power,
+                "url": self.url,
+                "status": self.status,
+                "message": self.message,
+            }
+        )
+
     def to_str(self):
         session = Session.object_session(self)
         rvalue = ""
         if self.url:
-            rvalue += "**Your SpellTable game is ready!**\n"
+            if self.message:
+                rvalue += f"{self.message}\n"
+            else:
+                rvalue += "**Your SpellTable game is ready!**\n"
             rvalue += f"{self.url}\n"
         else:
             rvalue += (
@@ -236,9 +277,10 @@ class Game(Base):
                 rvalue += f" _The average wait time is {delta}._\n"
             else:
                 rvalue += "\n"
-        rvalue += (
-            "\n🚨 When your game is ready I will send you another Direct Message! 🚨\n\n"
-        )
+            rvalue += (
+                "\n🚨 When your game is ready I will"
+                " send you another Direct Message! 🚨\n\n"
+            )
         players = ", ".join(sorted([f"<@{user.xid}>" for user in self.users]))
         rvalue += f"Players: {players}\n"
         if self.channel_xid:
